#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QShortcut>
#include <QDialog>
#include <QDialogButtonBox>
#include <QFormLayout>
#include <QLineEdit>
#include <QFile>
#include <QTextStream>
#include <QString>
#include <QCoreApplication>
#include <QSqlDatabase>
#include <iostream>
#include <QSqlQuery>
#include <QSqlError>
#include <QDebug>
#include <QSqlRecord>
#include <QSqlTableModel>
#include <QList>
#include <QSqlDatabase>
#include <iomanip>
#include <windows.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->lineEdit->setPlaceholderText("Enter the command");
    ui->lineEdit->setDisabled(true);
    connect(ui->pushButton, SIGNAL(clicked()), this, SLOT(ok_clicked()));
    connect(ui->pushButton_2, SIGNAL(clicked()), this, SLOT(clear_clicked()));
    connect(ui->listWidget, &QListWidget::itemClicked, this, [this](QListWidgetItem* item)
    {
        DBUpdate(item->text());
    });

    QShortcut *shortcut = new QShortcut(QKeySequence("Enter"), this); //пытался хоткей сделсть на ентер. Получилось на тот, что у нумпада
    QObject::connect(shortcut, SIGNAL(activated()), this, SLOT(ok_clicked())); //просто неудобно нажимать руками на ОК

}

void MainWindow::NewLogString(const QString& command) {
     ui->textEdit_log->append(command);
}

void MainWindow::ok_clicked() {
    QString command = ui->lineEdit->text();
    QSqlQuery query;

    bool cmd = query.exec(command);

     ui->textEdit_log->append(command);
     ui->textEdit_log->append("\n");

    if (!cmd) {
          ui->textEdit_log->append(query.lastError().text());
          DBUpdate("");
    }
    else {
        ui->textEdit_log->append("Command execution succesful!");
    }

}



void MainWindow::clear_clicked() {
    ui->lineEdit->clear();
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_actionExit_triggered()
{
    //вот тут вот перед выходом надо зыкрыть бд
    QApplication::quit();
}

struct config{
    QString SqlDatabase;
    QString HostName;
    QString DatabaseName;
    QString UserName;
    QString Password;
    QString Port;
};

config config;

void MainWindow::on_actionConnect_triggered()
{
    QDialog dlg(this);
    dlg.setWindowTitle(tr("Connect configuration"));
    dlg.isModal(); //делаем приоритетным окном
    dlg.setWindowFlags (dlg.windowFlags() & ~Qt::WindowContextHelpButtonHint); //убираем знак вопроса
    QLineEdit *SqlDatabase = new QLineEdit(&dlg);
    QLineEdit *HostName = new QLineEdit(&dlg);
    QLineEdit *DatabaseName = new QLineEdit(&dlg);
    QLineEdit *UserName = new QLineEdit(&dlg);
    QLineEdit *Password = new QLineEdit(&dlg);
    QLineEdit *Port = new QLineEdit(&dlg);

    QDialogButtonBox *btn_box = new QDialogButtonBox(&dlg);
    btn_box->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(btn_box, &QDialogButtonBox::accepted, &dlg, &QDialog::accept);
    connect(btn_box, &QDialogButtonBox::rejected, &dlg, &QDialog::reject);

    QFormLayout *layout = new QFormLayout();
    layout->addRow(tr("SqlDatabase:"), SqlDatabase);
    layout->addRow(tr("HostName:"), HostName);
    layout->addRow(tr("DatabaseName:"), DatabaseName);
    layout->addRow(tr("UserName:"), UserName);
    layout->addRow(tr("Password:"), Password);
    layout->addRow(tr("Port:"), Port);
    layout->addWidget(btn_box);

    //Пошла работа с файлами
    QFile file("config.txt");

    if (file.open(QIODevice::ReadOnly)) {
        QTextStream in(&file);
        SqlDatabase->setText(config.SqlDatabase = in.readLine());
        HostName->setText(config.HostName = in.readLine());
        DatabaseName->setText(config.DatabaseName = in.readLine());
        UserName->setText(config.UserName = in.readLine());
        Password->setText(config.Password = in.readLine());
        Port->setText(config.Port = in.readLine());
        file.close();
    }

    dlg.setLayout(layout);

    // В случае, если пользователь нажал "Ok".
    if(dlg.exec() == QDialog::Accepted) {

       ConnectToServer();

        if(file.open(QIODevice::WriteOnly | QIODevice::Text))
            {
                QTextStream out(&file);
                out << SqlDatabase->text() << "\n";
                out << HostName->text() << "\n";
                out << DatabaseName->text() << "\n";
                out << UserName->text() << "\n";
                out << Password->text() << "\n";
                out << Port->text() << "\n";
                file.close();
        }
    }
}

QSqlDatabase myDB;
void MainWindow::ConnectToServer() {
    myDB = QSqlDatabase::addDatabase(config.SqlDatabase);
    myDB.setHostName(config.HostName);
    myDB.setDatabaseName(config.DatabaseName);
    myDB.setUserName(config.UserName);
    myDB.setPassword(config.Password);
    myDB.setPort(config.Port.toInt());

    /*myDB = QSqlDatabase::addDatabase("QPSQL");
    myDB.setDatabaseName("test.db");*/

    if (!myDB.open()) {
        NewLogString("DB is not open!");
        NewLogString("ERR type:  ");
        NewLogString(myDB.lastError().text());
    }
    else {
        NewLogString("DB is open!");
        ui->lineEdit->setDisabled(false);
    }
    DBUpdate("");
}


void MainWindow::DBUpdate(const QString table_name) {

    QSqlTableModel *model = new QSqlTableModel(this);

    QStringList table_list = myDB.tables();
    ui->listWidget->clear();
    ui->listWidget->addItems(table_list);

    QStringList headers;
    if (table_name.size()) {
         model->setTable(table_name);
    }
    else if(table_list.size()) {
        model->setTable(table_list[0]);
    }
    for(int i = 0; i < model->columnCount(); i++)
    {
      headers.append(model->headerData(i, Qt::Horizontal).toString());
    }

    for(int i = 0, j = 0; i < model->columnCount(); i++, j++){
           model->setHeaderData(i,Qt::Horizontal,headers[j]);
    }

    ui->tableView->setModel(model);
    model->select();
}

